use warnings;
use strict;
use Test::More tests => 20;

use POWER::Event::Timer qw( :ALL );
use POWER::Utils;
use Time::HiRes qw( sleep time );
use List::MoreUtils qw( all );


# XXX       
# Epoll/Event-, ޣ    .   
#    ,    Σ  . :)
#          -  
#        .


# Different behaviours for clients: I/O schedule for next 2 seconds, where
# 1 mean IO happens (and so in/out timers will be reset) and 0 mean no IO.
my $Active_client_2sec = [
    1,  1,  1,  1,  1,  1,  1,  1,  1,  1,
    1,  1,  1,  1,  1,  1,  1,  1,  1,  1,
];
my $Semi_active_client_2sec = [
    1,  0,  1,  0,  1,  0,  1,  0,  1,  0,
    1,  0,  1,  0,  1,  0,  1,  0,  1,  0,
];
my $Active_client_1sec = [
    1,  1,  1,  1,  1,  1,  1,  1,  1,  1,
    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
];
my $Inactive_client = [
    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
];
# RULE: assign one of these behaviours to each client
my @ClientIO = (
    $Active_client_2sec,
    $Active_client_2sec,
    $Active_client_2sec,
    $Active_client_2sec,
    $Semi_active_client_2sec,
    $Semi_active_client_2sec,
    $Semi_active_client_2sec,
    $Active_client_1sec,
    $Active_client_1sec,
    $Inactive_client,
);
# RULE: accept that number of clients per second
my @Clients_accepted = (
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
     0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
);

# TEST: how many clients will exists at every second
my @Clients_num = (
    # no clients expired in first 4 seconds
    10, 20, 30, 40, 
    # in next 10 seconds all $Inactive_client become expired
                    49, 58, 67, 76, 85, 94,
    93, 92, 91, 90, 
    # in last 6 second 12 from 20 $Active_client_1sec become expired (2per1sec)
                    88, 86, 84, 82, 80, 78,
);

my @Clients;
sub close_client {
    my ($client) = @_;
    @Clients = grep { $_ != $client } @Clients;
    eval {
        stop_timer($client->{in});
        stop_timer($client->{out});
    };
}
for my $accepted (@Clients_accepted) {
    for my $i (0 .. $accepted-1) {
        my $client = {
            IO => [@{ $ClientIO[ $i % @ClientIO ] }],
        };
        $client->{in}  = set_timer({ for=>0.5, cb=>sub{close_client($client)} });
        $client->{out} = set_timer({ for=>0.5, cb=>sub{close_client($client)} });
        push @Clients, $client;
    }
    sleep 0.1;
    for my $client (@Clients) {
        my $io = shift @{ $client->{IO} };
        next unless $io;
        reset_timer($client->{in});
        reset_timer($client->{out});
    }
    run_callback(@$_) for get_expired_timers();
    is(@Clients, shift(@Clients_num));
}

